/** Compass interface Library for the STMicro LSM303D 3-axis magnetometer, 3-axis acceleromter
 *
 * Written by Eric Coyle
 * 
 * Modified by Abdurrahman Husnein for use on PIC32
 * Mod. Date: 12/2/2015
 *
 * Some references to Michael Shimniok's LSM303DLH library and 
 * Pololu sample library for LSM303DLH breakout by ryantm:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <plib.h>
#include <math.h>
#include "lsm303d.h"

#define _cs         LATAbits.LATA4 //pin 12 on PIC32
#define TRIS_cs     TRISAbits.TRISA4
#define _cs_high()  {LATASET = 16;}
#define _cs_low()   {LATACLR = 16;}

float _offset_x=0, _offset_y=0, _offset_z=0;
float _scale_x=1, _scale_y=1, _scale_z=1;


unsigned char comp_spiwrite(unsigned char c){
    unsigned char rd;
    Mode8();
    while (TxBufFullSPI1());
	WriteSPI1(c);
	while (SPI1STATbits.SPIBUSY); // wait for it to end of transaction
    rd = ReadSPI1();
    Mode16();
    return rd;
}

void comp_begin(){
    TRIS_cs = 0;
    _cs_high();
}


int whoami() {
    //talk to compass
    int me;
    _cs_low();
    //Send who am I (first bit must be 1 for read)
    comp_spiwrite(0x8F);
    me=comp_spiwrite(0x00);
    _cs_high();
    //Get who am I response
    //me = ReadSPI1();
      //done talking
    
    return me;
}
 
int comp_initialize() {
    int iam;
    
    //Check device
    iam=whoami();
    
    if (iam==73) {
        
        //CTRL0
        _cs_low();   //talk to compass
        comp_spiwrite(0x1F);
        comp_spiwrite(0x00);  //FIFO, High-pass set-up
        _cs_high();
        
        //set up accelerometer
        //CTRL1
        _cs_low();   //talk to compass
        comp_spiwrite(0x20);
        comp_spiwrite(0x57);  //50Hz, continuous update, all axes enabled
        _cs_high();   //done talking
        
        
        //CTRL2
        _cs_low();   //talk to compass
        comp_spiwrite(0x21);
        comp_spiwrite(0x00);  // 773Hz anti-alias filter, +/- 2g scale, self-test disabled, 4-wire SPI
        _cs_high();   //done talking
        
        //CTRL3
        
        //set up magnetometer
        //CTRL5
        _cs_low();   //talk to compass
        comp_spiwrite(0x24);
        comp_spiwrite(0xF0);  //temperature enabled, high resolution, 50 Hz, int2 disable, int1 disabled
        //mspi.write(0x70);  //temperature disabled, high resolution, 50 Hz, int2 disable, int1 disabled
        _cs_high();   //done talking
        
        
        //CTRL6 (Magnetic Scale)
        _cs_low();   //talk to compass
        comp_spiwrite(0x25);
        comp_spiwrite(0x20);  //+/-4g
        _cs_high();   //done talking
        
        //CTRL7 (filtering settings, and other)
        _cs_low();   //talk to compass
        comp_spiwrite(0x26);
        comp_spiwrite(0x00); //normal mode, keep other on
        _cs_high();   //done talking
        
        return 1;
    }
    else {
        //Not talking to right device
        return 0;
    }
}
 
void setOffset(float x, float y, float z)
{
    _offset_x = x;
    _offset_y = y;
    _offset_z = z;   
}
 
void setScale(float x, float y, float z)
{
    _scale_x = x;
    _scale_y = y;
    _scale_z = z;
}


short magnetometer(int axis) {
    unsigned char Data[2];
    short raw;
    if (axis==0) {
        _cs_low();   //lower cs to talk  
        comp_spiwrite(0x88);
        Data[0] = comp_spiwrite(0x00);
        _cs_high();   //done talking
        _cs_low();
        comp_spiwrite(0x89);
        Data[1] = comp_spiwrite(0x00);
        _cs_high();
    }
    else if (axis==1) {
        //Y-Axis
        _cs_low();
        comp_spiwrite(0x8A);
        Data[0] = comp_spiwrite(0x00);
        _cs_high();   //done talking
        _cs_low();
        comp_spiwrite(0x8B);
        Data[1] = comp_spiwrite(0x00);
        _cs_high();
    }
    else if (axis==2) {
        //Z-Axis
        _cs_low();
        comp_spiwrite(0x8C);
        Data[0] = comp_spiwrite(0x00);
        _cs_high();   //done talking
        _cs_low();
        comp_spiwrite(0x8D);
        Data[1] = comp_spiwrite(0x00);
        _cs_high();
    }
    raw = (short) ((((unsigned short)Data[1]) << 8) + 
                    ((unsigned short)Data[0]));
    return raw;
}
 
short accelerometer(int axis) { 
    char Data[2];
    short raw;
    if (axis==0) {
        //X-Axis
        _cs_low();
        comp_spiwrite(0xA8);
        Data[0] = comp_spiwrite(0x00);
        _cs_high();   //done talking
        _cs_low();
        comp_spiwrite(0xA9);
        Data[1] = comp_spiwrite(0x00);
        _cs_high();
    }
    else if (axis==1) {
        //Y-Axis
        _cs_low();
        comp_spiwrite(0xAA);
        Data[0] = comp_spiwrite(0x00);
        _cs_high();   //done talking
        _cs_low();
        comp_spiwrite(0xAB);
        Data[1] = comp_spiwrite(0x00);
        _cs_high();
    }
    else if (axis==2) {
        //Z-Axis
        _cs_low();
        comp_spiwrite(0xAC);
        Data[0] = comp_spiwrite(0x00);
        _cs_high();   //done talking
        _cs_low();
        comp_spiwrite(0xAD);
        Data[1] = comp_spiwrite(0x00);
        _cs_high();
    }
    raw = (short) ((((unsigned short)Data[1]) << 8) + 
                    ((unsigned short)Data[0]));
    return raw;
}
short x;
short y;
double heading()
{
    double head;
    x = magnetometer(0);
    y = -magnetometer(1);
    head= atan2( y,x) - 12*M_PI/180 + M_PI; //* 180/M_PI;
    return head;
}