/* Code rewritten from Adafruit Arduino library for the TFT
 *  by Syed Tahmid Mahbub
 * The TFT itself is Adafruit product 1480
 * Included below is the text header from the original Adafruit library
 *  followed by the code
 */

/***************************************************
  This is an Arduino Library for the Adafruit 2.2" SPI display.
  This library works with the Adafruit 2.2" TFT Breakout w/SD card
  ----> http://www.adafruit.com/products/1480

  Check out the links above for our tutorials and wiring diagrams
  These displays use SPI to communicate, 4 or 5 pins are required to
  interface (RST is optional)
  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
 ****************************************************/

#include "plib.h"
#include <GenericTypeDefs.h>

#define _dc         LATBbits.LATB7
#define TRIS_dc     TRISBbits.TRISB7
#define _dc_high()  {LATBSET = 128;}
#define _dc_low()   {LATBCLR = 128;}

#define _cs         LATBbits.LATB9
#define TRIS_cs     TRISBbits.TRISB9
#define _cs_high()  {LATBSET = 512;}
#define _cs_low()   {LATBCLR = 512;}

#define _rst        LATBbits.LATB2
#define TRIS_rst    TRISBbits.TRISB2
#define _rst_high() {LATBSET = 4;}
#define _rst_low()  {LATBCLR = 4;}

#define ILI9340_TFTWIDTH  240
#define ILI9340_TFTHEIGHT 320

#define ILI9340_NOP     0x00
#define ILI9340_SWRESET 0x01
#define ILI9340_RDDID   0x04
#define ILI9340_RDDST   0x09

#define ILI9340_SLPIN   0x10
#define ILI9340_SLPOUT  0x11
#define ILI9340_PTLON   0x12
#define ILI9340_NORON   0x13

#define ILI9340_RDMODE  0x0A
#define ILI9340_RDMADCTL  0x0B
#define ILI9340_RDPIXFMT  0x0C
#define ILI9340_RDIMGFMT  0x0A
#define ILI9340_RDSELFDIAG  0x0F

#define ILI9340_INVOFF  0x20
#define ILI9340_INVON   0x21
#define ILI9340_GAMMASET 0x26
#define ILI9340_DISPOFF 0x28
#define ILI9340_DISPON  0x29

#define ILI9340_CASET   0x2A
#define ILI9340_PASET   0x2B
#define ILI9340_RAMWR   0x2C
#define ILI9340_RAMRD   0x2E

#define ILI9340_PTLAR   0x30
#define ILI9340_MADCTL  0x36


#define ILI9340_MADCTL_MY  0x80
#define ILI9340_MADCTL_MX  0x40
#define ILI9340_MADCTL_MV  0x20
#define ILI9340_MADCTL_ML  0x10
#define ILI9340_MADCTL_RGB 0x00
#define ILI9340_MADCTL_BGR 0x08
#define ILI9340_MADCTL_MH  0x04

#define ILI9340_PIXFMT  0x3A

#define ILI9340_FRMCTR1 0xB1
#define ILI9340_FRMCTR2 0xB2
#define ILI9340_FRMCTR3 0xB3
#define ILI9340_INVCTR  0xB4
#define ILI9340_DFUNCTR 0xB6

#define ILI9340_PWCTR1  0xC0
#define ILI9340_PWCTR2  0xC1
#define ILI9340_PWCTR3  0xC2
#define ILI9340_PWCTR4  0xC3
#define ILI9340_PWCTR5  0xC4
#define ILI9340_VMCTR1  0xC5
#define ILI9340_VMCTR2  0xC7

#define ILI9340_RDID1   0xDA
#define ILI9340_RDID2   0xDB
#define ILI9340_RDID3   0xDC
#define ILI9340_RDID4   0xDD

#define ILI9340_GMCTRP1 0xE0
#define ILI9340_GMCTRN1 0xE1
/*
#define ILI9340_PWCTR6  0xFC

*/

// Color definitions
#define	ILI9340_BLACK   0x0000
#define	ILI9340_BLUE    0x001F
#define	ILI9340_RED     0xF800
#define	ILI9340_GREEN   0x07E0
#define ILI9340_CYAN    0x07FF
#define ILI9340_MAGENTA 0xF81F
#define ILI9340_YELLOW  0xFFE0
#define ILI9340_WHITE   0xFFFF

#define PBCLK 40000000 // peripheral bus clock
#define SPI_freq   10000000

#define tabspace 4 // number of spaces for a tab

#define dTime_ms PBCLK/2000
#define dTime_us PBCLK/2000000

#define tft_init_hw() {} // for compatibility with original version

void tft_spiwrite(UINT8 c);

/**
 * Transfer one byte via SPI
 * \param c The byte of data to transfer
 */
void tft_spiwrite8(UINT8 c);
inline void tft_spiwrite16(UINT16 c) __attribute__ ((always_inline));
void tft_writecommand(UINT8 c);
void tft_writecommand16(UINT16 c);
void tft_writedata(UINT8 c);
void tft_writedata16(UINT16 c);
void tft_begin(void);
void tft_setAddrWindow(UINT16 x0, UINT16 y0, UINT16 x1, UINT16 y1);
void tft_pushColor(UINT16 color);


/**
 * \Prototype void tft_drawPixel(INT16 x, INT16 y, UINT16 color)\n\n
 * Draw a pixel at location (x,y) with given color
 * \x x-coordinate of pixel to draw; top left of screen is x=0
 *              and x increases to the right
 * \y y-coordinate of pixel to draw; top left of screen is y=0
 *              and y increases to the bottom
 * \color 16-bit color value
 */
void tft_drawPixel(INT16 x, INT16 y, UINT16 color);

/**
 * \Prototype void tft_drawFastVLine(INT16 x, INT16 y, INT16 h, UINT16 color)\n\n
 * Draw a vertical line at location from (x,y) to (x,y+h-1) with color
 * \x  x-coordinate line to draw; top left of screen is x=0
 *              and x increases to the right
 * \y  y-coordinate of starting point of line; top left of screen is y=0
 *              and y increases to the bottom
 * \h  height of line to draw
 * \color:  16-bit color value
 */
void tft_drawFastVLine(INT16 x, INT16 y, INT16 h, UINT16 color);

/**
 * \Protoype void tft_drawFastHLine(INT16 x, INT16 y, INT16 w, UINT16 color) \n\n
 *
 * Draw a horizontal line at location from (x,y) to (x+w-1,y) with color
 * 
 * \x  x-coordinate starting point of line; top left of screen is x=0
 *              and x increases to the right
 * \y  y-coordinate of starting point of line; top left of screen is y=0
 *              and y increases to the bottom
 * \w  width of line to draw
 * \color:  16-bit color value
 */
void tft_drawFastHLine(INT16 x, INT16 y, INT16 w, UINT16 color);


/** \Prototype void tft_fillScreen(UINT16 color)\n\n
 *  Fill entire screen with given color
 * \color 16-bit color value
 */
void tft_fillScreen(UINT16 color);

/**
 * \Prototype void tft_fillRect(INT16 x, INT16 y, INT16 w, INT16 h, UINT16 color)\n\n
 *  Draw a filled rectangle with starting top-left vertex (x,y),
 *  width w and height h with given color
 * \x  x-coordinate of top-left vertex; top left of screen is x=0
 *              and x increases to the right
 * \y  y-coordinate of top-left vertex; top left of screen is y=0
 *              and y increases to the bottom
 * \w  width of rectangle
 * \h  height of rectangle
 * \color  16-bit color value
 */
void tft_fillRect(INT16 x, INT16 y, INT16 w, INT16 h, UINT16 color);


/**
 * \Prototype UINT16 tft_Color565(UINT8 r, UINT8 g, UINT8 b)\n\n
 * Pass 8-bit (each) R,G,B, get back 16-bit packed color
 * \r  8-bit R/red value from RGB
 * \g  8-bit g/green value from RGB
 * \b  8-bit b/blue value from RGB
 * \Returns 16-bit packed color value for color info
 */
UINT16 tft_Color565(UINT8 r, UINT8 g, UINT8 b);


void tft_setRotation(UINT8 m);

UINT16 _width, _height;

void delay_ms(unsigned long);

inline void tft_setAddrWindow_inline(UINT16 x0, UINT16 y0,
        UINT16 x1, UINT16 y1) __attribute__ ((always_inline));
